#!/usr/bin/perl
#
#  Add group to the LDAP database
#
use Net::LDAP;
use Getopt::Mixed "nextOption";
use File::Basename;

#
# The global variables
# 
my $global_gid;
my $ldapadd = 0;
my $unique  = 1;
my $jailed  = 0;
my $debug   = 0;
my $help    = 0;
my @groups;
my %id;
#
# Build hash of all used ID's on the system
#
my @passwdent = qx/getent passwd/;
my @groupent = qx/getent group/;
foreach (@passwdent, @groupent) {
	my $uid = (split(/:/, $_))[2];
	$id{$uid}=1 if ($uid > 0);
}
foreach (@passwdent) {
	my $gid = (split(/:/, $_))[3];
	$id{$uid}=1 if ($uid > 0);
}
foreach (@groupent) {
	my $group = (split(/:/, $_))[0];
	$groups{$group}=1;
}

my %config;
$config{ldap_host}      = "127.0.0.1";
$config{ldap_port}      = 389;
$config{ldap_version}   = 2;
$config{bind_dn}        = "cn=admin,dc=abode,dc=dk";
# Read ldap password if possible
if (open(PWF, "/etc/ldap.secret")) {
	@ldapsecrets=<PWF>;
	chomp(@ldapsecrets);
	$config{bind_pass} = $ldapsecrets[0];
} else {
	print "Unable to read admin password\n";
	exit(3);
}
$config{base_dn}        = "dc=abode,dc=dk";
$config{user_base_dn}   = "ou=People,dc=abode,dc=dk";
$config{group_base_dn}  = "ou=Group,dc=abode,dc=dk";
$config{user_filter}    = "(objectClass=posixAccount)";
$config{group_filter}   = "(objectClass=posixGroup)";
$config{base_dn}        = "dc=abode,dc=dk";
$config{hash}           = "{MD5}";
$config{mailspool}      = "/var/spool/mail";
$config{min_norm_group} = 2000;
$config{min_jail_group} = 3000;

#
# Usage message
#
sub usage
{
	print "\n";
	print "Usage: ", basename($0), " [OPTIONS] group\n";
	print "\n";
	print "Description:\n";
	print "\tAdd group to LDAP database.\n";
	print "\n";
	print "\tIf no group is given the program goes to interactive mode.\n";
	print "\n";
	print "Options:\n";
	print "\t-g : Group ID\n";
	print "\t-l : Add group to LDAP - even if it already exists\n";
	print "\t-o : Allow non-unique numbers\n";
	print "\t-j : Group belong in jailed area (gid > 3000)\n";
	print "\t-D : Enable debug messages\n";
	print "\t-? : This help (also --help)\n";
	print "\t-h : This help (also --help)\n";
	print "\n";
}
#
# Connect to the LDAP database
#
sub ldap_connect
{
	$ldap = Net::LDAP->new($config{ldap_host},
				port => $config{ldap_port},
				version => $config{ldap_version});
	$ldap->bind( dn => $config{bind_dn},
			password => $config{bind_pass},
	) or die "Could not bind to $config{ldap_host}: $!";
}

#
# Disconnect from the LDAP database
#
sub ldap_disconnect
{
	unbind $ldap ||
		die "Could not unbind from $config{ldap_host}: $!";
}

#
# Find free gid number 
#
sub id_first_free
{
	my $min_group;
	$min_group = $jailed
		? $config{min_jail_group}
		: $config{min_norm_group};
	while ($id{$min_group}) {
		$min_group++;
	}
	return($min_group);
}

sub group_check($$)
{
	my $group_name = $_[0];
	my $group_id = $_[1];
	print "Checking ", $group_name, " with ID=", $group_id, "\n"
		if ($debug);
	if ($id{$group_id} && $unique) {
		print "Failed check... GID=", $group_id, " already used.\n"
			if ($debug);
		return(0);
	}
	if ($groups{$group_name}) {
		if (!$ldapadd) {
			print "Failed check... Group=", $group_name,
				" already used.\n" if ($debug);
			return(0);
		}
	}
	return(1);
}

#
# Add the group to the LDAP directory
#
sub group_add
{
	if (!group_check($group_name, $global_gid)) {
		print "Unable to add ", $group_name, "...\n";
		return(0);
	}
	my $group_dn = "cn=" . $group_name . "," . $config{group_base_dn};
	print "Adding ", $group_name, " with gid=", $global_gid, "\n"
		if ($debug);
	print "Adding ", $group_dn, " to LDAP database\n" if ($debug);
	$ldap->add($group_dn,
		attr => [
			'objectClass' => ['posixGroup', 'top'],
			'cn' => $group_name,
			'userPassword' => '{MD5}x',
			'gidNumber' => $global_gid
			]
		);
}


#
# See if we are called correct, and parse commandline
#
Getopt::Mixed::init("j l o g=i D h ? help>h");
while (($option, $value) = nextOption()) {
	$ldapadd    = 1       if ($option eq "l");
	$unique     = 0       if ($option eq "o");
	$jailed     = 1       if ($option eq "j");
	$debug      = 1       if ($option eq "D");
	$help       = 1       if ($option eq "h");
	$help       = 1       if ($option eq "?");
	$global_gid = $value  if ($option eq "g");
}

if ($help) {
	usage;
	exit;
}
$group_name = $ARGV[0];
if (!$group_name) {
	print "Group to add : ";
	$group_name = <>;
	chomp($group_name);
	if (!$global_gid && $group_name) {
		$global_gid = id_first_free();
		print "Group ID (", $global_gid, ") : ";
		$new_gid = <>;
		chomp($new_gid);
		if ($new_gid) {
			if ($new_gid > 0) {
				if (!$id{$new_gid} || !$unique) {
					$global_gid = $new_gid;
				} else {
					print "ID=", $new_gid, " already used.";
					print "Aborting...\n";
					exit(0);
				}
			} else {
				print "Nonsense entered : ", $new_gid, "\n";
				print "Aborting...\n";
				exit(0);
			}
		}
	}
}

exit if (!$group_name);
if (!$global_gid) {
	$global_gid = id_first_free();
}
print "Group name : ", $group_name, "\n" if ($debug);
print "Group ID   : ", $global_gid, "\n" if ($debug);

&ldap_connect;

#
# add group
#
group_add;

&ldap_disconnect;

