#!/usr/bin/perl
#
#  Test script... Attempting to connect to LDAP database
#
use Net::LDAP;
use Getopt::Mixed "nextOption";

my $debug = 0;
my $help  = 0;
#
# The global variables
# 
my %config;
$config{ldap_host}     = "127.0.0.1";
$config{ldap_port}     = 389;
$config{ldap_version}  = 2;
$config{bind_dn}       = "cn=admin,dc=abode,dc=dk";
# Read ldap password if possible
if (open(PWF, "/etc/ldap.secret")) {
	@ldapsecrets=<PWF>;
	chomp(@ldapsecrets);
	$config{bind_pass} = $ldapsecrets[0];
} else {
	print "Unable to read admin password\n";
	exit(3);
}
$config{base_dn}       = "dc=abode,dc=dk";
$config{user_base_dn}  = "ou=People,dc=abode,dc=dk";
$config{group_base_dn} = "ou=Group,dc=abode,dc=dk";
$config{user_filter}   = "(objectClass=posixAccount)";
$config{group_filter}  = "(objectClass=posixGroup)";
$config{base_dn}       = "dc=abode,dc=dk";
$config{hash}          = "{MD5}";
$config{mailspool}     = "/var/spool/mail";

#
# Usage message
#
sub usage
{
	print "\n";
	print "Usage: ", basename($0), " group\n";
	print "\n";
	print "Description:\n";
	print "\tRemove group from LDAP database.\n";
	print "\n";
	print "\tIf no group is given the program goes to interactive mode.\n";
	print "\n";
	print "Options:\n";
	print "\t-D : Enable debug messages\n";
	print "\t-? : This help (also --help)\n";
	print "\t-h : This help (also --help)\n";
	print "\n";
}
#
# Connect to the LDAP database
#
sub ldap_connect
{
	$ldap = Net::LDAP->new($config{ldap_host},
				port => $config{ldap_port},
				version => $config{ldap_version});
	$ldap->bind( dn => $config{bind_dn},
			password => $config{bind_pass},
	) or die "Could not bind to $config{ldap_host}: $!";
}

#
# Disconnect from the LDAP database
#
sub ldap_disconnect
{
	unbind $ldap ||
		die "Could not unbind from $config{ldap_host}: $!";
}

#
# Check how many users have a group as primary group
#
sub group_user_check($)
{
	my $group_name  = $_[0];
	my $membercount = 0;
	#
	# Find the groups gid
	#
	my $groups = $ldap->search( base => $config{base_dn},
		filter => $config{group_filter} ) 
		or die "could not search with base ", $config{base_dn},
		       " and filter ", $config{group_filter};
	foreach $group ($groups->all_entries) {
		$g_name = $group->get_value("cn");
		next if ($g_name ne $group_name);
		$g_id = $group->get_value("gidNumber");
		break;
	}
	#
	# Check if there are others that have this group as primary
	#
	my $users = $ldap->search( base => $config{base_dn},
		filter => $config{user_filter} )
		or warn "Could not search with base ", $config{base_dn},
			" and filter ", $config{group_filter};
	foreach my $user ($users->all_entries) {
		my $u_name = $user->get_value("uid");
		my $u_gid = $user->get_value("gidNumber");
		if ($u_gid == $g_id) {
			$membercount++;
			print $u_name, " is member of ", $group_name, "\n"
				if ($debug);
		}
	}
	return($membercount);
}

sub group_del($)
{
	my $group = $_[0];
	my $group_dn = "cn=" . $group . "," . $config{group_base_dn};
	print "Attempting to delete ", $group_dn, "\n" if ($debug);
	$num_primaries = group_user_check($group);
	if ($num_primaries > 0) {
		print "Unable to delete group ", $group, "\n";
		print "There are ", $num_primaries, 
		      " user(s) that have this group as default\n";
		return(0);
	}
	print "Deleting LDAP group ", $group, "\n" if ($debug);
	$ldap->delete( dn => $group_dn ) || die "Unable to delete ",
		$group_dn, ": ", $!;
}

#
# See if we are called correct, and parse commandline
#
Getopt::Mixed::init("D h ? help>h");
while (($option, $value) = nextOption()) {
	$debug   = 1   if ($option eq "D");
	$help    = 1   if ($option eq "h");
	$help    = 1   if ($option eq "?");
}

if ($help) {
	usage;
	exit;
}
$group_name = $ARGV[0];
if (!$group_name) {
	print "Group to remove : ";
	$group_name = <>;
	chomp($group_name);
}
exit if (!$group_name);

&ldap_connect;

#
# Remove group
#
group_del($group_name);

&ldap_disconnect;

