#!/usr/bin/perl
#
#  Test script... Attempting to connect to LDAP database
#
use File::Basename;
use File::Copy;
use File::Path;
use Getopt::Mixed "nextOption";
use Net::LDAP;
use Time::Local;

#
# Define global variables to be populated by command line or interactively
#
my $login_name;
my $comment            = "";
my $user_id            = 0;
my $group_id           = 0;
my $unique             = 1;
my $additional_groups;
my $home_directory;
my $user_shell;
my $make_home          = 0;
my $template_directory = "/etc/skel";
my $password;
my $debug              = 0;
my $help               = 0;
my $optioncount        = 0;
my @group_members;

my @passwdent = qx/getent passwd/;
my @groupent = qx/getent group/;
foreach (@passwdent) {
	my $uid = (split(/:/, $_))[2];
	$userids{$uid}=1;
}
foreach (@passwdent, @groupent) {
	my $uid = (split(/:/, $_))[2];
	$id{$uid}=1 if ($uid > 0);
}
foreach (@passwdent) {
	my $gid = (split(/:/, $_))[3];
	$id{$uid}=1 if ($uid > 0);
}
foreach (@groupent) {
	my $group = (split(/:/, $_))[0];
	my $gid   = (split(/:/, $_))[2];
	$groups{$group}=1;
	$groupids{$gid}=1;
}

#
# The config variable
# 

my %config;
$config{ldap_host}     = "127.0.0.1";
$config{ldap_port}     = 389;
$config{ldap_version}  = 2;
# Read ldap password if possible
if (open(PWF, "/etc/ldap.secret")) {
	@ldapsecrets=<PWF>;
	chomp(@ldapsecrets);
	$config{bind_pass} = $ldapsecrets[0];
} else {
	print "Unable to read admin password\n";
	exit(3);
}
$config{base_dn}        = "dc=abode,dc=dk";
$config{user_base_dn}   = "ou=People," . $config{base_dn};
$config{group_base_dn}  = "ou=Group," . $config{base_dn};
$config{user_filter}    = "(objectClass=posixAccount)";
$config{group_filter}   = "(objectClass=posixGroup)";
$config{bind_dn}        = "cn=admin," . $config{base_dn};
$config{hash}           = "{MD5}";
$config{mailspool}      = "/var/spool/mail";
$config{norm_home}      = "/home";
$config{norm_shell}     = "/bin/bash";
$config{norm_min_group} = 2000;
$config{max_user_id}    = 10000;
$config{max_group_id}   = 10000;
$config{template}       = "/etc/skel";
$config{encryption}     = "{MD5}";
$config{email_domain}   = "hoejte.dk";

#
# Usage message
#
sub usage
{
	print "Usage: ", basename($0), "\t[-u uid [-o]] [-g group] [-G group,...] \n";
	print "\t\t[-d home] [-s shell] [-c comment] [-m [-k template]]\n";
	print "\t\t[-p passwd] name\n";
	print "\n";
	print "Description:\n";
	print "\tAdd user to the LDAP database.\n";
	print "\n";
	print "Options:\n";
	print "\t-u : User ID\n";
	print "\t-o : Allow non-unique user ID's (not login names)\n";
	print "\t-g : Default Group ID\n";
	print "\t-G : Additional groups\n";
	print "\t-d : Home directory base\n";
	print "\t-s : Login shell\n";
	print "\t-c : Comment (usually the full name of user)\n";
	print "\t-m : Make home directory\n";
	print "\t-k : Template directory\n";
	print "\t-p : Password (plaintext)\n";
	print "\t-? : This help (also --help)\n";
	print "\t-h : This help (also --help)\n";
	print "\n";
}
#
# Connect to the LDAP database
#
sub ldap_connect
{
	print "bind_dn = " . $config{bind_dn} . "\n";
	$ldap = Net::LDAP->new($config{ldap_host},
				port => $config{ldap_port},
				version => $config{ldap_version});
	$ldap->bind( dn => $config{bind_dn},
			password => $config{bind_pass},
	) or die "Could not bind to $config{ldap_host}: $!";
}

#
# Disconnect from the LDAP database
#
sub ldap_disconnect
{
	unbind $ldap ||
		die "Could not unbind from $config{ldap_host}: $!";
}

#
# Get the first free GID/UID
#
sub id_first_free
{
	my $min_group;
	$min_group = $config{norm_min_group};
	while ($id{$min_group}) {
		$min_group++;
	}
	return($min_group);
}

#
# Check if a user exists
#
sub user_exists($)
{
	my $login  = $_[0];
	my $retval = 0;
	print "ue: Check if ", $login, " exists\n" if ($debug);
	my $users = $ldap->search( base => $config{base_dn},
		filter => $config{user_filter} ) 
		or die "could not search with base ", $config{base_dn},
		       " and filter ", $config{user_filter};
	foreach my $user ($users->all_entries) {
		my $uid = $user->get_value("uid");
		next if ($uid !~ m/^$login$/);
		print "ue: User ", $login, " exists\n" if ($debug);
		$retval=1;
	}
	print "ue: Return value : ", $retval, "\n" if ($debug);
	return($retval)
}

#
# Check if group exists
#
sub group_exists($)
{
	my $group  = $_[0];
	my $retval = 0;
	my $groups = $ldap->search( base => $config{base_dn},
		filter => $config{group_filter} ) 
		or die "could not search with base ", $config{base_dn},
		       " and filter ", $config{group_filter};
	foreach my $groupname ($groups->all_entries) {
		my $group_name = $groupname->get_value("cn");
		next if ($group_name ne $group);
		$retval = 1;
		print "ge: Group ", $group, " exists\n" if ($debug);
	}
	return($retval);
}

#
# Check if group already exists.
#
sub group_check($$)
{
	my $group_name = $_[0];
	my $group_id = $_[1];
	print "gc: Checking ", $group_name, " with ID=", $group_id, "\n"
		if ($debug);
	if ($groupids{$group_id} && $unique) {
		print "gc: Failed check... GID=", $group_id, " already used.\n"
			if ($debug);
		return(0);
	}
	if ($groups{$group_name}) {
		if (!$ldapadd) {
			print "gc: Failed check... Group=", $group_name,
				" already used.\n" if ($debug);
			return(0);
		}
	}
	return(1);
}

#
# Add the group to the LDAP directory
#
sub group_add($$)
{
	my $group_name = $_[0];
	my $group_id = $_[1];
	my $group_dn = "cn=" . $group_name . "," . $config{group_base_dn};
	if (!group_check($group_name, $group_id)) {
		print "Unable to add ", $group_name, "...\n";
		return(0);
	}
	print "ga: Adding ", $group_name, " with gid=", $group_id, "\n"
		if ($debug);
	print "ga: Adding ", $group_dn, " to LDAP database\n" if ($debug);
	$ldap->add($group_dn,
		attr => [
			'objectClass' => ['posixGroup', 'top'],
			'cn' => $group_name,
			'userPassword' => '{MD5}x',
			'gidNumber' => $group_id
			]
		);
}

sub group_user_add($$)
{
	my $group_name = $_[0];
	my $memberuid  = $_[1];
	print "gua: Adding ", $memberuid, " to ", $group_name, "\n" if ($debug);
	my $groups = $ldap->search( base => $config{base_dn},
		filter => $config{group_filter} ) 
		or die "could not search with base ", $config{base_dn},
		       " and filter ", $config{group_filter};
	foreach my $groupname ($groups->all_entries) {
		my $g_name = $groupname->get_value("cn");
		next if ($g_name ne $group_name);
		$g_dn = $groupname->dn;
		print "gua: Now adding to ", $g_dn, "\n" if ($debug);
		@group_members = $groupname->get_value("memberUid");
		push(@group_members, $login_name);
		print "-", join(",", @group_members), "\n" if ($debug);
		$ldap->modify($g_dn, replace => [ 
			'memberUid' => [ @group_members ] ]
		);
		return(1);
	}
	return(0);
	
}
#
# Add user to LDAP 
#
sub user_add
{
	my $user_dn = "uid=" . $login_name . "," . $config{user_base_dn};
	my $user_mail = $login_name . "@" . $config{email_domain};
	print "ua: Adding ", $login_name, " with uid=", $user_id, "\n"
		if ($debug);
	print "ua: Adding ", $user_dn, " to LDAP database\n" if ($debug);
	#
	# Add the user
	#
	if (!$comment) {
		print "ua: Making default comment\n" if ($debug);
		$comment = $login_name;
	}
	$ldap->add($user_dn,
		attrs => [
			'objectClass' => ['account', 'posixAccount', 
					'top', 'shadowAccount',
					],
			'uid' => $login_name,
			'cn' => "$comment",
			'userPassword' => $cryptpass,
			'loginShell' => $user_shell,
			'uidNumber' => $user_id,
			'gidNumber' => $group_id,
			'homeDirectory' => $home_directory,
			'gecos' => $comment
			]
		);
	if (user_exists($login_name)) {
		print "ua: Added ", $login_name, " successfully\n" if ($debug);
	} else {
		print "ua: Failed to add ", $login_name, "\n" if ($debug);
		exit(1);
	}
	#
	# Add the primary group
	#
	if (!$groupids{$group_id}) {
		#
		# We need to make a new primary group
		#
		group_add($login_name, $group_id);
	}
	#
	# Add user as a member to the secondary groups - if any
	#
	my @secondaries = split(/,/, $additional_groups);
	foreach (@secondaries) {
		print "ua: Adding ", $user_id, " to ", $_, "\n" if ($debug);
		group_user_add($_, $login_name);
	}
	#
	# Create home directory if it does not exist.
	#
	if ($make_home) {
		print "ua: Creating home directory\n" if ($debug);
		mkdir($home_directory) or 
			warn "Unable to create ", $home_directory;
		chown($user_id, $group_id, $home_directory) or 
			warn "Unable to set permissions on ", $home_directory;
		chmod(0755, $home_directory) or
			warn "Unable to set mode on ", $home_directory;
		if (opendir(TEMPLATE, $template_directory)) {
			while (my $direntry = readdir(TEMPLATE)) {
				next if (-d $direntry);
				$home_entry = $home_directory . "/" . $direntry;
				$skel_entry = $template_directory . "/" . $direntry;
				print "ua: Copying $skel_entry to $home_entry\n"
					if ($debug);
				if (copy($skel_entry, $home_entry)) {
					chown($user_id, $group_id, $home_entry);
				} else {
					print "Unable to copy $skel_entry to",
						" $home_directory\n";
				}
			}
			closedir(TEMPLATE);
		}
	}
}
#
# See if we are called correct, and parse commandline
#
Getopt::Mixed::init("u=i o g=i G=s d=s c=s m k=s p=s s=s l D h ? help>h");
while (($option, $value) = nextOption()) {
	$optioncount++;
	$user_id            = $value  if ($option eq "u");
	$unique             = 0       if ($option eq "o");
	$group_id           = $value  if ($option eq "g");
	$additional_groups  = $value  if ($option eq "G");
	$home_directory     = $value  if ($option eq "d");
	$user_shell         = $value  if ($option eq "s");
	$comment            = $value  if ($option eq "c");
	$make_home          = 1       if ($option =~ m/^[mj]$/);
	$template_directory = $value  if ($option eq "k");
	$password           = $value  if ($option eq "p");
	$debug              = 1       if ($option eq "D");
	$optioncount--                if ($option eq "D");
	$help               = 1       if ($option eq "h");
	$help               = 1       if ($option eq "?");
}

#
# Print usage message if that's what's specified.
#
if ($help) {
	usage;
	exit;
}

#
# Get login name
#
if ($ARGV[0]) {
	$login_name = $ARGV[0];
	shift;
	$optioncount++;
}
&ldap_connect;

#
# Start by getting the login-name
#
if (!$optioncount) {
	print "User to add          : ";
	$login_name = <>;
	chomp($login_name);
	if (!$login_name) {
		print "Need a user name to continue\n";
		exit(1);
	}
	print "Real name            : ";
	$comment = <>;
	chomp($comment);
} else {
	if (!$login_name) {
		usage;
		exit(1);
	}
}
#
# Check if login already exists
#
if (user_exists($login_name)) {
	print "User ", $login_name, " already exists\n";
	exit(1);
}

#
# Test if we were given options. If yes, then we suppose that default
# values is to be used for all options NOT specified.
#
print $optioncount, " options given on the command line\n" if ($debug);

#
# Next we get the User ID
#
$response = "";
if (!$optioncount) {
	#
	# Interactive (no options given)
	#
	my $default_id = id_first_free;
	while (!$response && !$user_id) {
		print "User ID (", $default_id, ")       : ";
		$response = <>;
		chomp($response);
		my $testresponse = $response;
		$testresponse =~ s/[^0-9]//g;
		if ($testresponse ne $response
			|| (!$response && $response ne "")
			|| $response > $config{max_user_id}) {
			print "User ID is a number between 1 and ",
				$config{max_user_id}, "\n";
			$response = "";
			next;
		}
		if ($response eq "") {
			$user_id = $default_id;
		} else {
			if ($userids{$response} && $unique) {
				print "User ID=", $response,
					" is already used\n";
				$response = "";
				next;
			}
			$user_id = $response
		}

	}
} else {
	# non interactive (options given)
	if (!$user_id) {
		#
		# -u was not one of the options, or it was set to 0 
		# which is not permissible.
		#
		$user_id = id_first_free;
	} else {
		# user id from commandline
		if ($userids{$user_id} && $unique) {
			print "User ID=", $user_id, " is already used\n";
			exit(1);
		}
	}
}

#
# Primary group id
#
$response = "";
if (!$optioncount) {
	# Interactive
	my $default_id;
	if ($groupids{$user_id}) {
		$default_id = id_first_free;
	} else {
		$default_id = $user_id;
	}
	while (!$response && !$group_id) {
		print "Group ID (", $default_id, ")      : ";
		$response = <>;
		chomp($response);
		my $testresponse = $response;
		$testresponse =~ s/[^0-9]//g;
		if ($testresponse ne $response
			|| (!$response && $response ne "")
			|| $response > $config{max_group_id}) {
			print "Group ID is a number between 1 and ",
				$config{max_group_id}, "\n";
			$response = "";
			next;
		}
		if ($response eq "") {
			$group_id = $default_id;
		} else {
			$group_id = $response
		}
	}
} else {
	# Automatic assignment
	if (!$group_id) {
		$group_id = $groupids{$user_id} ? id_first_free : $user_id;
	}
}

#
# Supplementary groups
#
$response = "";
if (!$optioncount) {
	# Ask for supplementary groups
	# Check the supplied groups
	my $groupcheck = 0;
	my @subgroups;
	while (!$groupcheck) {
		print "Supplementary groups : ";
		$response = <>;
		chomp($response);
		@subgroups = split(/[,:; ]/, $response);
		$groupcheck=1;
		foreach (@subgroups) {
			if (!group_exists($_)) {
				print "Group ", $_, " does not exist in LDAP\n";
				$groupcheck = 0;
			}
		}
	}
	$additional_groups = join(",", @subgroups);
} else {
	# Check specified groups (if any)
	my @subgroups = split(/[,:; ]/, $additional_groups);
	my $groupcheck = 1;
	foreach (@subgroups) {
		if (!group_exists($_)) {
			print "Group ", $_, " does not exist in LDAP\n";
			$groupcheck = 0;
		}
	}
	if (!$groupcheck) {
		print "One or more additonal groups was not found in\n";
		print "the LDAP database.\n";
		exit(1);
	}
}

#
# Set the home directory
#
$response = "";
if (!$optioncount) {
	# Interactive
	while (!$response) {
		print "Default home directory base = ", $config{norm_home}, "\n";
		print "Home directory base  : ";
		$response = <>;
		chomp($response);
		if ($response eq "") {
			$response = $config{norm_home};
		}
		if (! -d $response) {
			print "Directory ", $response, " does not exist.\n";
			$response="";
			next;
		}
		my ($hdev, $hino, $hmode, $hnlink, $huid, $hgid, $hrdev,
			$hsize, $hatime, $hmtime, $hctime, $hblksize,
			$hblocks) = stat($response);
		if ($huid != 0) {
			print $response, " is not owned by root (uid=0)\n";
			$response="";
			next;
		}
		$home_directory = $response;
	}
} else {
	# Automatic
	if (!$home_directory) {
		$home_directory = $config{norm_home};
	}
	if (! -d $home_directory) {
		print "Directory ", $home_directory, " does not exist.\n";
		exit(1);
	}
	my ($hdev, $hino, $hmode, $hnlink, $huid, $hgid, $hrdev,
		$hsize, $hatime, $hmtime, $hctime, $hblksize,
		$hblocks) = stat($home_directory);
	if ($huid != 0) {
		print $home_directory, " is not owned by root (uid=0)\n";
		exit(1);
	}
}
$home_directory =~ s#[/]+$##g;
$home_directory .= "/" . $login_name;


#
# Set the user shell
#
$response = "";
if (!$optioncount) {
	# Interactive
	my $default_shell = $config{norm_shell};
	while (!$response) {
		print "Shell (", $default_shell, ") : ";
		$response = <>;
		chomp($response);
		if ($response eq "") {
			$response = $default_shell;
		}
		if (! -x $response || ! -f $response) {
			print $response, " is not an executable file\n";
			$response="";
			next;
		}
	}
	$use_shell = $response;
} else {
	# Automatic
	if (!$user_shell) {
		$user_shell = $config{norm_shell};
	} else {
		if ( ! -x $user_shell || ! -f $user_shell ) {
			print $user_shell, " is not an executable\n";
			exit(1);
		} 
	}
}

#
# Template directory is NOT an interactive option
#
if (! -d $template_directory) {
	print $template_directory, " is not a directory, and can\n";
	print "therefore not be used as a skeleton directory.\n";
	exit(1);
}

#
# Password is ALWAYS interactive, unless a password was entered on the
# command line
#
$response="x";
if (!$password) {
	# Interactive. Password not given
	my $validate = "y";
	my $pass = 0;
	while (!$response || $response ne $validate) {
		if ($pass) {
			print "Password mismatch. Try again.\n";
		}
		system("stty -echo");
		print "Password  : ";
		$response = <>;
		chomp($response);
		print "\n";
		print "Validate  : ";
		$validate = <>;
		chomp($validate);
		print "\n";
		$pass=1;
		system("stty echo");
	}
	$password = $response
}
#
# Password encryption
#
$cryptpass = `slappasswd -h $config{encryption} -s $password`;
chomp($cryptpass);
if ($debug) {
	print "----------------------------------------------------------\n";
	print "Login name           : ", $login_name, "\n";
	print "Real name            : ", $comment, "\n";
	print "User ID              : ", $user_id, "\n";
	print "Group ID             : ", $group_id, "\n";
	print "Supplementary groups : ", $additional_groups, "\n";
	print "Home directory       : ", $home_directory, "\n";
	print "User shell           : ", $user_shell, "\n";
	print "Make home directory  : ", $make_home, "\n";
	print "Template directory   : ", $template_directory, "\n";
	print "Password             : ", $password, "\n";
	print "Encrypted password   : ", $cryptpass, "\n";
	print "==========================================================\n";
}

#
# Do the add thing
#
user_add();

&ldap_disconnect;


