#!/usr/bin/perl
#
#  Test script... Attempting to connect to LDAP database
#
use Net::LDAP;
use Getopt::Mixed "nextOption";
use File::Path;
use File::Basename;

my $debug = 1;
my $help  = 0;
#
# The global variables
# 
my %config;
my $global_dn;
my $global_gid;
my $global_uid;
my $global_home;
my @global_groups;
my $cleanup = 0;

$config{ldap_host}     = "127.0.0.1";
$config{ldap_port}     = 389;
$config{ldap_version}  = 2;
$config{bind_dn}       = "cn=admin,dc=abode,dc=dk";
# Read ldap password if possible
if (open(PWF, "/etc/ldap.secret")) {
	@ldapsecrets=<PWF>;
	chomp(@ldapsecrets);
	$config{bind_pass} = $ldapsecrets[0];
} else {
	print "Unable to read admin password\n";
	exit(3);
}
$config{base_dn}       = "dc=abode,dc=dk";
$config{user_base_dn}  = "ou=People,dc=abode,dc=dk";
$config{group_base_dn} = "ou=Group,dc=abode,dc=dk";
$config{user_filter}   = "(objectClass=posixAccount)";
$config{group_filter}  = "(objectClass=posixGroup)";
$config{base_dn}       = "dc=abode,dc=dk";
$config{hash}          = "{MD5}";
$config{mailspool}     = "/var/spool/mail";

#
# Usage message
#
sub usage
{
	print "\n";
	print "Usage: ", basename($0), " [OPTIONS] [NAME]\n";
	print "\n";
	print "Description:\n";
	print "\tRemove user from LDAP database.\n";
	print "\tOptionally removes mailspool and homedirectory.\n";
	print "\n";
	print "\tIf no name is given the program goes to interactive mode.\n";
	print "\n";
	print "Options:\n";
	print "\t-r : Remove home directory and mailspool\n";
	print "\t-D : Enable debug messages\n";
	print "\t-? : This help (also --help)\n";
	print "\t-h : This help (also --help)\n";
	print "\n";
}
#
# Connect to the LDAP database
#
sub ldap_connect
{
	$ldap = Net::LDAP->new($config{ldap_host},
				port => $config{ldap_port},
				version => $config{ldap_version});
	$ldap->bind( dn => $config{bind_dn},
			password => $config{bind_pass},
	) or die "Could not bind to $config{ldap_host}: $!";
}

#
# Disconnect from the LDAP database
#
sub ldap_disconnect
{
	unbind $ldap ||
		die "Could not unbind from $config{ldap_host}: $!";
}

#
# Check if a user exists
#
sub user_check($)
{
	my $login = $_[0];
	my $retval=0;
	my $users = $ldap->search( base => $config{base_dn},
		filter => $config{user_filter} ) 
		or die "could not search with base ", $config{base_dn},
		       " and filter ", $config{user_filter};
	foreach my $user ($users->all_entries) {
		my $uid = $user->get_value("uid");
		next if ($uid !~ m/^$login$/);
		$retval++ if ($uid =~ m/^$login$/);
		$global_dn = $user->dn;
		$global_uid = $user->get_value("uidNumber");
		$global_gid = $user->get_value("gidNumber");
		$global_home= $user->get_value("homeDirectory");
	}
	print "global_dn = $global_dn\n" if ($debug);
	print "Found ", $retval, " users matching ", $login, "\n" if ($debug);
	return($retval)
}

#
# Check what groups a user is a member of
#
sub user_group_check($)
{
	my $login = $_[0];
	my $retval=0;
	my $groups = $ldap->search( base => $config{base_dn},
		filter => $config{group_filter} ) 
		or die "could not search with base ", $config{base_dn},
		       " and filter ", $config{group_filter};
	@global_groups=();
	foreach my $group ($groups->all_entries) {
		my $g_name = $group->get_value("cn");
		my $g_id = $group->get_value("gidNumber");
		my @g_mem = grep(/^$login$/, $group->get_value("memberUid"));
		my $g_count = @g_mem;
		my $g_dn = $group->dn;
		if ($g_count > 0) {
			push (@global_groups, $g_dn);
		}
	}
}

#
# Delete user from groups
#
sub user_group_del($)
{
	my $login = $_[0];
	user_group_check($login);
	foreach my $group_dn (@global_groups) {
		printf("Deleting %s from %s\n", $login, $group_dn) if ($debug);
		$ldap->modify($group_dn, delete => { 'memberUid' => $login } );
	}
}

#
# Check how many users are in a group
#
sub group_user_check($)
{
	my $group_name  = $_[0];
	my $membercount = 0;
	#
	# Find the groups gid
	#
	my $groups = $ldap->search( base => $config{base_dn},
		filter => $config{group_filter} ) 
		or die "could not search with base ", $config{base_dn},
		       " and filter ", $config{group_filter};
	foreach $group ($groups->all_entries) {
		$g_name = $group->get_value("cn");
		next if ($g_name ne $group_name);
		$g_id = $group->get_value("gidNumber");
		break;
	}
	#
	# Check if there are others that have this group as primary
	#
	my $users = $ldap->search( base => $config{base_dn},
		filter => $config{user_filter} )
		or warn "Could not search with base ", $config{base_dn},
			" and filter ", $config{group_filter};
	foreach my $user ($users->all_entries) {
		my $u_name = $user->get_value("uid");
		my $u_gid = $user->get_value("gidNumber");
		if ($u_gid == $g_id) {
			$membercount++;
			print $u_name, " is member of ", $group_name, "\n"
				if ($debug);
		}
	}
	return($membercount);
}

sub group_del($)
{
	my $group = $_[0];
	my $group_dn = "cn=" . $group . "," . $config{group_base_dn};
	print "gd: Attempting to delete ", $group_dn, "\n" if ($debug);
	$num_primaries = group_user_check($group);
	if ($num_primaries > 0) {
		print "Unable to delete group ", $group, "\n";
		print "There are ", $num_primaries, 
		      " user(s) that have this group as default\n";
		return(0);
	}
	print "gd: Deleting LDAP group ", $group, "\n" if ($debug);
	$ldap->delete( dn => $group_dn ) || die "Unable to delete ",
		$group_dn, ": ", $!;
}
#
# Delete user (only root can change other users)
#
sub user_del($)
{
	my $login = $_[0];
	my $delsuccess = 0;
	print "Attemtpting to delete user ", $login, "\n" if ($debug);
	my $num_users = user_check($login);
	if ($num_users != 1) {
		print $num_users, " users found that matches ", $login, "\n" 
			if ($debug);
		return(0);
	}
	print "Deleting LDAP user : ", $login, "\n" if ($debug);
	print "global_dn = $global_dn\n" if ($debug);
	$ldap->delete( dn => $global_dn ) || die "Unable to delete ",
		$global_dn, ": ", $!;
	print "Deleting LDAP group ", $login, " if there are no members\n"
		if ($debug);
	group_del($login);
	print "Deleting ", $login, " from all LDAP groups\n" if ($debug);
	user_group_del($login);
	if ($cleanup) {
		print "Cleaning up home directory and mail-spool\n"
			if ($debug);
		if ( $global_home && -e $global_home ) {
			print "Attempting to remove ", $global_home, "\n" 
				if ($debug);
			if (rmtree($global_home)) {
				print $global_home, " removed.\n"
					if ($debug);
			}
		}
		my $user_mailspool = $config{mailspool} . "/" . $login;
		if ( -e $user_mailspool ) {
			print "Removing spool file ", $user_mailspool, "\n";
			if (unlink($user_mailspool)) {
				print $user_mailspool, " removed.\n"
					if ($debug);
			}
		}
	}
	print "User ", $login, " deleted.\n" if ($debug);
}

#
# See if we are called correct, and parse commandline
#
Getopt::Mixed::init("r D h ? help>h");
while (($option, $value) = nextOption()) {
	$cleanup = 1   if ($option eq "r");
	$debug   = 1   if ($option eq "D");
	$help    = 1   if ($option eq "h");
	$help    = 1   if ($option eq "?");
}

if ($help) {
	usage;
	exit;
}
$login_name = $ARGV[0];
if (!$login_name) {
	print "User to remove : ";
	$login_name = <>;
	chomp($login_name);
}
exit if (!$login_name);

&ldap_connect;

#
# Remove user
#
user_del($login_name);

&ldap_disconnect;

